<?php
namespace FM\Controller;

use FM\Controller\FMCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use FM\Service\Fm00003000ChecklistService;
use FM\Traits\TResponse;

class Fm00003000ChecklistController extends FMCrud {

    use TResponse;
    protected $ChecklistService;

    public function __construct($crudService, $environment, Fm00003000ChecklistService $ChecklistService) {
            parent::__construct($crudService, $environment);
            $this->ChecklistService = $ChecklistService;
    }

    public function getAtividadesChecklist(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->getAtividadesChecklist($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function getDadosChecklist(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->getDadosChecklist($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function salvarChecklist(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->salvarChecklist($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function getChecklist(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->getChecklist();
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    // FUNÇÕES DA TAB DE ITENS E CRITÉRIOS
    public function getItensChecklist(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->getItensChecklist();
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function salvarItemChecklist(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->salvarItemChecklist($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function getCriteriosChecklist(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->getCriteriosChecklist($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function salvarTarefaChecklist(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->salvarTarefaChecklist($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function getTarefasChecklist(Request $request, Response $response) {
        try {
            $data = $this->ChecklistService->getTarefasChecklist($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function inativarTarefasChecklist(Request $request, Response $response) {
        try {
            $this->ChecklistService->inativarTarefasChecklist($request->getParameters());
        } catch (Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    // FUNÇÕES DA TAB ATIVIDADE E CRITÉRIOS
    public function salvarAtividadeCriterio(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->salvarAtividadeCriterio($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function getAtividadesCriterios(Request $request, Response $response) {
        try {
            // $request->getParameters()
                $data = $this->ChecklistService->getAtividadesCriterios();
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function inativarAtividadesCriterios(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->inativarAtividadesCriterios($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    // FUNÇÕES DA TAB DE ESCOPO
    public function getEscoposServicos(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->getEscoposServicos();
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function salvarEscopoServico(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->salvarEscopoServico($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function inativarEscopos(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->inativarEscopos($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    // FUNÇÕES PARA MODAL DE EDIÇÃO

    public function getTipoLocalModalEdicao(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->getTipoLocalModalEdicao($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function getItensModalEdicao(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->getItensModalEdicao($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function getAtividadesModalEdicao(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->getAtividadesModalEdicao($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function getDadosChecklistEdicao(Request $request, Response $response) {
        try {
                $data = $this->ChecklistService->getDadosChecklistEdicao($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

    public function edicaoChecklist(Request $request, Response $response) {
        try {
           
                $data = $this->ChecklistService->edicaoChecklist($request->getParameters());
                $this->sendResponse($data, 200);
            } catch (Exception $ex) {
                $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
            }
    }

}