/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/*                                                                            */
/* Qualidade de Fornecimento                                                  */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome: 8590_QFOSTP_APURA_QUALIF_FORN                                        */
/*                                                                            */
/* Descrio:                                                                 */
/*  Faz a apurao da qualificao dos fornecedores                           */
/*                                                                            */
/*                                                                            */
/* Criao:                                                                   */
/* 31/01/2001 - Joo Fernando dos Reis Corra                                 */
/*                                                                            */
/*                                                                            */
/******************************************************************************/
/******************************************************************************/
/* PARAMETROS:                                                                */
/* P_DTVALIDADE - Nova data de validade                                       */
/* P_ALTERARVAL - Indica se a data de validade ser auterada ou no           */
/* P_DTINIOCOR  - Data inicial da ocorrncia                                  */
/* P_DTFINOCOR  - Data final da ocorrncia                                    */
/* P_CDFILIAL   - Lista de Filiais                                            */
/* P_CDFORNECED - Lista de Fornecedores                                       */
/* P_TODAS      - Todas as Filiais                                            */
/* P_TODOS      - Todos os Fornecedores                                       */
/******************************************************************************/
IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE
           O.NAME = 'APURA_QUALIF_FORN' AND
	   TYPE = 'P' 		  AND
	   U.UID = O.UID          AND
	   U.NAME = USER)

   DROP PROCEDURE APURA_QUALIF_FORN
GO

CREATE PROCEDURE APURA_QUALIF_FORN
(
   @P_CDFILIPROC VARCHAR(4),
   @P_CDOPERADOR VARCHAR(12),
   @P_DTINIOCOR  DATETIME,
   @P_DTFINOCOR  DATETIME,
   @P_CDFILIAL   VARCHAR(2000),
   @P_CDFORNECED VARCHAR(2000),
   @P_TODAS      VARCHAR,
   @P_TODOS      VARCHAR
)
AS
   DECLARE @V_NRMAXOCORR   NUMERIC;
   DECLARE @V_DTINIOCOR    DATETIME;
   DECLARE @V_DTFINOCOR    DATETIME;
   DECLARE @V_MAXNRDIAS    NUMERIC;
   DECLARE @V_NRLOGAPURQFO VARCHAR(20);
   DECLARE @V_CDQUAANT     VARCHAR(2);
   DECLARE @V_STRINGAUX    VARCHAR;

   DECLARE @V_FORN_CDFORNECED   VARCHAR(21);
   DECLARE @V_FORN_CDTIPRISCO   VARCHAR(2);
   DECLARE @V_FORN_NRMAXOCORR   VARCHAR(3);
   DECLARE @V_FORN_NRDIASOCORR  VARCHAR(3);
   DECLARE @V_FORN_CDQUALIF     VARCHAR(2);
   DECLARE @V_FORN_DTVALQUALIF DATETIME;

   DECLARE @V_DATA_DTPERINI DATETIME;
   DECLARE @V_DATA_DTPERFIN DATETIME;
BEGIN
   /*Diminui o intervalo da Apurao, por questo de performance, para intervalos
     informados grandes.*/
   SELECT
      @V_DTINIOCOR = CONVERT(DATETIME, CONVERT(VARCHAR, MIN(DTOCORRMOV), 103)),
      @V_DTFINOCOR = CONVERT(DATETIME, CONVERT(VARCHAR, MAX(DTOCORRMOV), 103)),
      @V_MAXNRDIAS = CONVERT(NUMERIC, MAX(NRDIASOCORR))
   FROM
      MOVOCORR, TOLECONF, OCORRENCIA
   WHERE
      (CHARINDEX(MOVOCORR.CDFORNECED, @P_CDFORNECED) <> 0 OR @P_TODOS = 'T') AND
      (CHARINDEX(MOVOCORR.CDFILIAL, @P_CDFILIAL)     <> 0 OR @P_TODAS = 'T') AND
      (MOVOCORR.CDGRPOCOR  = OCORRENCIA.CDGRPOCOR)  AND
      (MOVOCORR.CDOCORR    = OCORRENCIA.CDOCORR)    AND
      (TOLECONF.CDTIPRISCO = OCORRENCIA.CDTIPRISCO) AND
      CONVERT(DATETIME, CONVERT(VARCHAR, DTOCORRMOV, 103)) BETWEEN @P_DTINIOCOR AND @P_DTFINOCOR;

   IF (@V_DTFINOCOR - @V_DTINIOCOR) < @V_MAXNRDIAS
   BEGIN
      SET @V_DTINIOCOR = @P_DTINIOCOR;
      SET @V_DTFINOCOR = @P_DTFINOCOR;
   END;
   DECLARE C_FORN CURSOR FOR/*Retorna todos os fornecedores que tiveram ocorrncia*/
       SELECT
          MOVOCORR.CDFORNECED,
          TOLECONF.CDTIPRISCO,
          TOLECONF.NRMAXOCORR,
          TOLECONF.NRDIASOCORR,
          TOLECONF.CDQUALIF,
          FORNECEDOR.DTVALQUALIF
       FROM
          MOVOCORR,
          OCORRENCIA,
          TOLECONF,
          FORNECEDOR
       WHERE
          (FORNECEDOR.CDFORNECED = MOVOCORR.CDFORNECED) AND
          (CHARINDEX(MOVOCORR.CDFORNECED, @P_CDFORNECED) <> 0 OR @P_TODOS = 'T') AND
          (CHARINDEX(MOVOCORR.CDFILIAL, @P_CDFILIAL)     <> 0 OR @P_TODAS = 'T') AND
          (MOVOCORR.CDGRPOCOR  = OCORRENCIA.CDGRPOCOR)  AND
          (MOVOCORR.CDOCORR    = OCORRENCIA.CDOCORR)    AND
          (TOLECONF.CDTIPRISCO = OCORRENCIA.CDTIPRISCO) AND
          (CONVERT(DATETIME, CONVERT(VARCHAR, MOVOCORR.DTOCORRMOV, 103)) BETWEEN @V_DTINIOCOR AND @V_DTFINOCOR)
       GROUP BY
          MOVOCORR.CDFORNECED,
          TOLECONF.NRDIASOCORR,
          TOLECONF.CDTIPRISCO,
          TOLECONF.NRMAXOCORR,
          TOLECONF.CDQUALIF,
          FORNECEDOR.DTVALQUALIF;
   OPEN C_FORN;
   FETCH NEXT FROM C_FORN INTO @V_FORN_CDFORNECED,
                               @V_FORN_CDTIPRISCO,
                               @V_FORN_NRMAXOCORR,
                               @V_FORN_NRDIASOCORR,
                               @V_FORN_CDQUALIF,
                               @V_FORN_DTVALQUALIF;
   WHILE (@@FETCH_STATUS = 0)
   BEGIN
      DECLARE C_DATA CURSOR FOR/*Separa o intervalo de datas em perodos, de acordo com*/
          SELECT   /*o nmero de dias de ocorrncia.*/
             DTDATA AS DTPERINI,
             DTDATA + CONVERT(NUMERIC, @V_FORN_NRDIASOCORR) - 1 AS DTPERFIN
          FROM
             DATA
          WHERE
             DTDATA BETWEEN @V_DTINIOCOR AND @V_DTFINOCOR - CONVERT(NUMERIC, @V_FORN_NRDIASOCORR) + 1;
      OPEN C_DATA;
      FETCH NEXT FROM C_DATA INTO @V_DATA_DTPERINI, @V_DATA_DTPERFIN;
      WHILE (@@FETCH_STATUS = 0)
      BEGIN
         SET @V_NRMAXOCORR = 0;
         SELECT
            @V_NRMAXOCORR = COUNT(*)  /*Nmero de ocorrncias neste perodo.*/
         FROM
            MOVOCORR, OCORRENCIA
         WHERE
            MOVOCORR.CDGRPOCOR = OCORRENCIA.CDGRPOCOR AND
            MOVOCORR.CDOCORR = OCORRENCIA.CDOCORR     AND
            OCORRENCIA.CDTIPRISCO = @V_FORN_CDTIPRISCO AND
            MOVOCORR.CDFORNECED = @V_FORN_CDFORNECED  AND
            CONVERT(DATETIME, CONVERT(VARCHAR, DTOCORRMOV, 103)) BETWEEN @V_DATA_DTPERINI AND @V_DATA_DTPERFIN;
         IF @V_NRMAXOCORR > @V_FORN_NRMAXOCORR
         BEGIN
            SET @V_CDQUAANT = NULL;
            SELECT
               @V_CDQUAANT = CDQUALIF/* Retorna a qulificao do fornecedor */
            FROM                       /* antes da apurao.*/
               FORNECEDOR
            WHERE
               CDFORNECED = @V_FORN_CDFORNECED;

            UPDATE               /*Se o nmero de ocorrncias for MAIOR que*/
               FORNECEDOR        /*o permitido ento updata a qualificao*/
            SET                  /*e a data.*/
               CDQUALIF    = @V_FORN_CDQUALIF,
               DTVALQUALIF = ISNULL(@V_FORN_DTVALQUALIF, CONVERT(DATETIME, CONVERT(VARCHAR, GETDATE(), 103))) + CONVERT(NUMERIC, @V_FORN_NRDIASOCORR),
               IDFORNAUTO  = 'N'
            WHERE
               FORNECEDOR.CDFORNECED = @V_FORN_CDFORNECED;

            SET @V_STRINGAUX = 'LOGAPURQFO' + @P_CDFILIPROC;
            EXEC NOVO_CODIGO @V_STRINGAUX, @P_SEQUENCIAL = @V_NRLOGAPURQFO OUTPUT, @P_QTDE = 1;
            INSERT INTO LOGAPURQFO (CDFILIAL,
                                    NRLOGAPURQFO,
                                    DTHRAPURQFO,
                                    DTINIAPURQFO,
                                    DTFINAPURQFO,
                                    CDOPERADOR,
                                    CDFORNECED,
                                    CDQUAANT,
                                    CDQUADEP)
                        VALUES (@P_CDFILIPROC,
                                SUBSTRING(@V_NRLOGAPURQFO, 11, 10),
                                GETDATE(),
                                @P_DTINIOCOR,
                                @P_DTFINOCOR,
                                @P_CDOPERADOR,
                                @V_FORN_CDFORNECED,
                                @V_CDQUAANT,
                                @V_FORN_CDQUALIF);

            INSERT INTO OCOAPURQFO (CDFILIAL,
                                    NRLOGAPURQFO,
                                    CDFILILANCMO,
                                    NROCORRMOV)
                        (SELECT
                            @P_CDFILIPROC,
                            SUBSTRING(@V_NRLOGAPURQFO, 11, 10),
                            MOVOCORR.CDFILILANCMO,
                            MOVOCORR.NROCORRMOV
                         FROM
                            MOVOCORR, OCORRENCIA
                         WHERE
                            (CHARINDEX(MOVOCORR.CDFILIAL, @P_CDFILIAL) <> 0 OR @P_TODAS = 'T') AND
                            (CONVERT(DATETIME, CONVERT(VARCHAR, MOVOCORR.DTOCORRMOV, 103)) BETWEEN @V_DTINIOCOR AND @V_DTFINOCOR) AND
                            (MOVOCORR.CDGRPOCOR = OCORRENCIA.CDGRPOCOR)  AND
                            (MOVOCORR.CDOCORR = OCORRENCIA.CDOCORR)      AND
                            (OCORRENCIA.CDTIPRISCO = @V_FORN_CDTIPRISCO) AND
                            (MOVOCORR.CDFORNECED = @V_FORN_CDFORNECED));

            BREAK;
         END;
         FETCH NEXT FROM C_DATA INTO @V_DATA_DTPERINI, @V_DATA_DTPERFIN;
      END;
      CLOSE C_DATA;
      DEALLOCATE C_DATA;
      FETCH NEXT FROM C_FORN INTO @V_FORN_CDFORNECED,
                                  @V_FORN_CDTIPRISCO,
                                  @V_FORN_NRMAXOCORR,
                                  @V_FORN_NRDIASOCORR,
                                  @V_FORN_CDQUALIF,
                                  @V_FORN_DTVALQUALIF;
   END;
   CLOSE C_FORN;
   DEALLOCATE C_FORN;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'APURA_QUALIF_FORN','4.05.126';
GO

